unit FSClientDataSet;

interface

uses
  SysUtils, Classes, DB, DBClient, Dialogs, UnitMensagemErro, Variants;

type
  TFSClientDataSet = class(TClientDataSet)
  private
    { Private declarations }
    MsgErroInserir, MsgErroEditar, MsgErroApagar, MsgBanco : String;
    MsgMostrar, AutoAP : Boolean;
    procedure ValidaClientDataSet(Evento : Integer);
    procedure MostraMensagem(Msg1, Msg2 : String);
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Post; override;
    procedure internaldelete; override;
    constructor Create(AOwner : TComponent); override;
  published
    { Published declarations }
    property MensagemErroInserir : string read MsgErroInserir write MsgErroInserir;
    property MensagemErroEditar  : string read MsgErroEditar write MsgErroEditar;
    property MensagemErroApagar  : string read MsgErroApagar write MsgErroApagar;
    property MensagemBanco       : string read MsgBanco write MsgBanco;
    property MostrarMensagens    : boolean read MsgMostrar write MsgMostrar;
    property AutoApplyUpdates    : boolean read AutoAP write AutoAP;
  end;

procedure Register;

implementation


constructor TFSClientDataSet.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    MsgErroInserir   := 'Erro ao inserir o registro.';
    MsgErroEditar    := 'Erro ao editar o registro.';
    MsgErroApagar    := 'Erro ao apagar o registro.';
    MsgBanco         := '';
    MostrarMensagens := True;
    AutoAP           := True;
end;

procedure TFSClientDataSet.ValidaClientDataSet(Evento : Integer);
// Evento: 1-Post, 2-Delete
var
   f : integer;
   ValoresAnteriores : Array of Variant;
begin

   // Se houve mudanas no ClientDataSet, aplica updates.
   if ChangeCount > 0 then
   begin

      // Busca valores Anteriores
      setLength(ValoresAnteriores, FieldCount);
      for f := 0 to FieldCount - 1 do
         if Fields[f].asString = '' then
            ValoresAnteriores[f] := null
         else
            ValoresAnteriores[f] := Fields[f].NewValue;

      // 1-Post
      if Evento = 1 then
      begin
        if ApplyUpdates(0) > 0 then
        begin
          if UpdateStatus = usInserted then
          begin
            if MsgMostrar then
               MostraMensagem(MsgErroInserir, MsgBanco);
            CancelUpdates;
            Last;
            Append;
          end
          else
          begin
            if MsgMostrar then
               MostraMensagem(MsgErroEditar, MsgBanco);
            CancelUpdates;
            Edit;
          end;
          for f := 0 to FieldCount - 1 do
            Fields[f].Value := ValoresAnteriores[f];
          Abort;
        end
        else
           Refresh;
      end;

      // 2-Delete
      if Evento = 2 then
      begin
         if ApplyUpdates(0) > 0 then
         begin
            if MsgMostrar then
               MostraMensagem(MsgErroApagar, MsgBanco);
            CancelUpdates;
         end;
         Refresh;
 //        Abort;
      end;

   end;
end;

procedure TFSClientDataSet.MostraMensagem(Msg1, Msg2 : String);
begin
    try
       MensagemErro := TMensagemErro.Create(nil);
       MensagemErro.Lab_Msg1.Caption := Msg1;
       MensagemErro.Lab_Msg2.Caption := Msg2;
       MensagemErro.ShowModal;
    finally
       FreeandNil(MensagemErro);
    end;
end;

procedure TFSClientDataSet.Post;
begin
    inherited Post;
    if AutoAP then
       ValidaClientDataSet(1);
end;

procedure TFSClientDataSet.InternalDelete;
begin
    inherited InternalDelete;
    if AutoAP then
       ValidaClientDataSet(2);
end;


procedure Register;
begin
  RegisterComponents('Data Access', [TFSClientDataSet]);
end;

end.
